/******************************************************************

        Command:  DDINSERT -- A dialog base INSERT/MINSERT/RINSERT
                              command

        Purpose:  Provide a more convenient way to insert a drawing
                  or block.

        Programmed by Kang, Hsin-Min.

        Released at Mar 15, 1998 -- Initial release
        Revised at  Apr 22, 1998 -- Include Japenese Text in SJIS codes
        Revised at  Jul 16, 1998 -- Fix a data consistancy problem if
                                    the user tries to modify an MINSERT
                                    into an RINSERT.

        Copyright(c) 1998 TCAM Development House, Taipei.
        All rights Reserved.

        Status: TwinCAD Standard Command, with full support from
                TCAM Development House.

        Program Interface through exbuf_io():

        Input:  %%DDINSERT -MODIFYSINGLE Modify single insert only

        Output: ##DDINSERT CANCEL        if user cancels
                ##DDINSERT OK            if the operation succeeded
                ##DDINSERT FAIL          if operation fails

    Comments:

        The CHINSERT command is implemented to call this command
        in single insert modification mode.

*******************************************************************/

#define BOOL    int
#define TRUE    1
#define FALSE   0
#define EOF     -1

#define FINSERT         0x100

#define SECTIONNAME     "DDInsert"
#define MAINWINDOW      "MainWindow"

#define MARKON  0x8000          /* DOS version compatible marking value */
#define MARKOFF 0x80ff          /* DOS version compatible marking value */
#define DATACOLOR       0xf1
#define DATACOLOR1      0xf7
#define FMTVALUE        "%8.3f"
#define FNAMESIZE       0x2e
#define FBLKSIZE        0x0f

#define DOCREATE        1
#define DOPICKUP        2
#define DOMODIFY        3
#define DOPREVIEW       4
#define DOPICKPOS       5
#define DOPICKCENTER    6

#define ppangle(ps,pe)   atan2((pe).y-(ps).y,(pe).x-(ps).x)
#define polar(p,r,a)     p+P(r,(a)A)

int     IsWin;
int     nSupport=identify("wnd_resize");
// Certain functions are supported only in Windows version and after
// wnd_resize(), so checking this will help to identify these supports.

int     widMain;        // Main Dialog Window ID
int     widVport;       // Sub Window for view port
int     nMain[6]={4,23|0x0800,10,70,0x3270,0x8000};
int     nTatrb=0x270;
int     nSatrb=0x270;
//
char    szBtn[10][10];  // For all common buttons (8 chars)
char    szText[40][72]; // Our display text buffer, see SetMsgText()
                        // at the end of this file

#define FILEMAGICID     0x1a434241L
#define MAXBNAME        16
#define MAXBLOCK        512             // Maximum can be 1024

int     nBlock=0;
int     nBlkNo=0;
char    szBlkName[MAXBLOCK][MAXBNAME];
char    szSetFile[128];

int     nExp=0;
int     nMult=0;
int     nPolar=0;
int     nCmd=0;
char    szBlock[64];
char    szName[64]="";
POINT   pBase={0.,0.};
POINT   pScale={1.,1.};
double  dAngle, dBAngle;
int     nRow=1, nCol=1;
POINT   pSpace={0.,0.};
POINT   pCenter={0.,0.};
double  dSpan=360.;
double  dSAngle=0.;
double  dRadius=0;
int     nCount=1;
int     nRot=0;

ENTITY  eTarget=ent_null();

int     mUndoAll=-1;

#define NOTPRESENT      -99

int     bBlock;

int     bName;
int     bBase;
int     bScale;
int     bAngle;
int     bExp=NOTPRESENT;
int     bMult;
int     bRect;
int     bRow, bCol;
int     bSpace;
int     bBAngle;
int     bCenter;
int     bSpan, bSAngle;
int     bCount;
int     bRot;
int     bRad;
int     bCmd=NOTPRESENT;

int     bPick;
int     bCreate=NOTPRESENT;
int     bCancel, bHelp;
int     bModify=NOTPRESENT;


#define BEINGCALLED     1
#define CREATEONLY      2
#define MODIFYONLY      4
#define MODIFYSINGLE    8

int     nCallFlag=0;    // How DDINSERT is called.

COMMAND DDInsert()
{
        int     nRet;
        IsWin = 1;
        switch(1) {  // Test if we are in TwinCAD ...
          case !identify("os_type"):
          case (os_type()<0):   // Must be in DOS version!
                IsWin = 0;
                break;
        }
        mUndoAll = set_undo(1);
        SetMsgText();

        CheckCall();    // Check the program interface
//
//      nCallFlag = BEINGCALLED|MODIFYSINGLE;     // Set for testing purpose!
//
        Initialize();

        if ( nCallFlag & MODIFYSINGLE ) {
                eTarget = gete(szText[24]/*"\nSelect the block instance to modify: "*/,FINSERT);
                if ( !ent_ok(eTarget) ) {
                        exbuf_io(0,"##DDINSERT CANCEL",128);
                        exit(1);
                }
                PickInsData(eTarget,1);
        }
        userbrk(0);     // Turn off User Break
        nRet = FALSE;
        for(;;) {
                switch(DoDialog()) {
                  case DOPICKPOS:   DoPickPos(); continue;
                  case DOPICKCENTER:DoPickCen(); continue;
                  case DOPICKUP:    DoPickup(); continue;
                  case DOPREVIEW:   DoPreview(); continue;
                  case DOCREATE:    nRet = DoCreate(); break;
                  case DOMODIFY:    nRet = DoModify(); break;
                  default:
                        nRet = FALSE;
                        break;
                }
                break;
        }
        if ( !nRet ) {
                undo(mUndoAll);
                if ( nCallFlag & BEINGCALLED )
                        exbuf_io(0,"##DDINSERT CANCEL",128);
        }
        else if ( nCallFlag & BEINGCALLED ) {
                exbuf_io(0,"##DDINSERT OK",128);
        }
}
//
void    DoPickPos()
{
        pBase = getp(szText[25]/*"\nPlease designate the base insertion point: "*/,pBase);
        dSAngle = ppangle(pCenter,pBase);
        dRadius = pCenter,pBase;
}
//
void    DoPickCen()
{
        pCenter = getp(szText[26]/*"\nPlease designate the rotation center point: "*/,pCenter);
        dSAngle = ppangle(pCenter,pBase);
        dRadius = pCenter,pBase;
}
//
//      DoCreate() -- Process the creation of associate hatch upon the
//                    [Create] button.
//
BOOL    DoCreate()
{
        int     cmdecho=@cmdecho;
        int     nRet;
        char    cmdbuf[256];

        cmdbuf = "INSERT ";
        if ( nMult ) {
                if ( nPolar ) cmdbuf = "RINSERT ";
                else          cmdbuf = "MINSERT ";
        }
        //
        //      We need to use string expression to bypass the possible
        //      '\\' code conflict with the menu script system.
        //
        s1 = "";
        if ( nExp ) s1 += "*";
        if ( !nExp || szName=="" )      // If Exploded, don't create the
                s1 += szBlock;          // block for external drawing
        if ( szName != "" ) s1 += "="+szName;
        cmdbuf += "@!s1^M";
        if ( nCmd && !(nCallFlag & (MODIFYSINGLE|MODIFYONLY)) ) {
                // Use command line parameters
                @cmdecho = 0;
                nRet = command(cmdbuf);
                @cmdecho = cmdecho;
                return nRet;
        }
        p1 = pBase;         cmdbuf += "p1 ";
        v1 = pScale.x;  v2 = pScale.y;
        cmdbuf += "v1 v2 ";
        v3 = dAngle;
        cmdbuf += "v3 ";
        if ( nMult ) {
                if ( nPolar ) {
                        p2 = pCenter;  cmdbuf += "p2 ";
                        i1 = nCount;
                        v4 = dSpan;
                        cmdbuf += "i1 v4 ";
                        if ( nRot ) cmdbuf += "Y ";
                        else        cmdbuf += "N ";
                }
                else {
                        i1 = nRow; i2 = nCol;
                        cmdbuf += "i1 i2 ";
                        v4 = pSpace.y; v5 = pSpace.x;
                        cmdbuf += "v4 v5 ";
                        v6 = dBAngle;
                        cmdbuf += "v6 ";
                }
        }
        @cmdecho = 0;
        command(cmdbuf);
        @cmdecho = cmdecho;
        return TRUE;
}
//
//      DoModify() -- Process the modification of block instance upon
//                    the [Modify] button.
//
BOOL    DoModify()
{
        ENTDATA data, data1;
        int     cmdecho = @cmdecho;
        int     nModified;
        if ( !ent_ok(eTarget) ) return;
        data = ent_read(eTarget);
        data1 = ent_read(eTarget);
        nModified = 0;
        data.insert.pbase = pBase;
        data.insert.pscale = pScale;
        data.insert.pspace = pSpace;
        data.insert.angle = dtr(dAngle);
        data.insert.beta = dtr(dBAngle);
        data.insert.colno = nMult?nCol:1;
        data.insert.rowno = nMult?nRow:1;
        data.insert.name = szBlock;
        if ( nMult && nPolar ) {
                data.insert.rowno = 1;
                data.insert.rowno |= 0x8000;
                data.insert.pbase = pCenter;
                if ( !nRot ) data.insert.rowno |= 0x4000;
                data.insert.colno = nCount;
                data.insert.beta = dtr(dSAngle);
                data.insert.gamma = dRadius;
        }
        nModified = 0;
        if ( data.insert.pbase != data1.insert.pbase ) nModified++;
        if ( data.insert.pscale != data1.insert.pscale ) nModified++;
        if ( data.insert.pspace != data1.insert.pspace ) nModified++;
        if ( data.insert.angle != data1.insert.angle ) nModified++;
        if ( data.insert.beta != data1.insert.beta ) nModified++;
        if ( data.insert.colno != data1.insert.colno ) nModified++;
        if ( data.insert.rowno != data1.insert.rowno ) nModified++;
        if ( data.insert.name != data1.insert.name ) nModified++;
        if ( data.insert.gamma != data1.insert.gamma ) nModified++;
        if ( nModified ) {
                //
                //      ent_write() does not rebuild the internal link
                //      and extent data of the INSERT entity if one should
                //      have modified the block name in reference and the
                //      other parameters.  This problem has been fixed
                //      since the function ent_state() is supported.
                //
                if ( identify("ent_state") && (szName!="") )
                        ent_write(eTarget,data);
                else {
                        @cmdecho = 0;
                        e1 = eTarget;
                        command("ERASE e1 ^M^M^M");
                        @cmdecho = cmdecho;
                        DoCreate();
                }
        }
        return TRUE;
}
//
void    DoPickup()
{
        ENTITY  ePick;
        POINT   pBaseSave;
        ePick = gete(szText[27]/*"\nPlease select the block instance to sample: "*/,FINSERT);
        PickInsData(ePick,0);
}
//
void    PickInsData(ENTITY ent, int org)
{
        ENTDATA data;

        if ( !ent_ok(ent) ) return;
        data = ent_read(ent);

        if ( org ) pBase = data.insert.pbase;
        pScale = data.insert.pscale;
        if ( pScale.x == 0 ) pScale.x = 1;
        if ( pScale.y == 0 ) pScale.y = 1;
        pSpace = data.insert.pspace;
        dAngle = rtd(data.insert.angle);
        dBAngle = rtd(data.insert.beta);
        nCol = data.insert.colno&0x7fff;
        nRow = data.insert.rowno&0x7fff;
        szBlock = data.insert.name;
        nBlock = search(szBlock,szBlkName,nBlkNo);
        if ( nBlock == -1 ) nBlock = nBlkNo-1;
        nMult = 0;
        if ( data.insert.rowno&0x8000 ) {       // Rotational
                nMult = 1;
                nPolar = 1;
                pCenter = data.insert.pbase;
                nCount = data.insert.colno;
                nRot = (data.insert.rowno&0x4000)?0:1;
                dSAngle = rtd(data.insert.beta);
                dRadius = data.insert.gamma;
                if ( org ) pBase = polar(pCenter,dRadius,dSAngle);
                else pCenter += pBase - polar(pCenter,dRadius,dSAngle);
                dBAngle = 0;
        }
        else if ( nCol > 1 || nRow > 1 ) {
                nMult = 1;
                nPolar = 0;
        }
        szName = "";
        return;
}

int     DoDialog()
{
        int     nBtn;
        int     nRet;

        OpenDialog();
        UpdateDialog();

        nRet = -1;
        for(;;) {
                nBtn = wnd_gbtn(0);
                switch(nBtn) {
                  case -3:
                  case -2:
                  case -1:
                  case  0:      nRet = -1;       break;
                  case bPick:   nRet = DOPICKUP; break;
                  case bCreate: nRet = DOCREATE; break;
                  case bModify: nRet = DOMODIFY; break;
                  case bBlock:  SetBName();     continue;
                  case bBlock+1:SelBName();     continue;
                  case bName:   SetName(nBtn);   continue;
                  case bAngle:  SetAngle(nBtn); continue;
                  case bBAngle: SetBAngle(nBtn); continue;
                  case bSAngle: SetSAngle(nBtn); continue;
                  case bSpan:   SetSpan(nBtn);  continue;
                  case bRad:    SetRadius(nBtn); continue;
                  case bScale:
                  case bScale+1:SetScale(nBtn); continue;
                  case bBase:
                  case bBase+1:  SetPos(nBtn);   continue;
                  case bBase+2:  nRet = DOPICKPOS; break;
                  case bCenter:
                  case bCenter+1: SetCenter(nBtn); continue;
                  case bCenter+2: nRet = DOPICKCENTER; break;
                  case bSpace:
                  case bSpace+1:SetSpace(nBtn); continue;
                  case bRow:    SetRow(nBtn);   continue;
                  case bCol:    SetCol(nBtn);   continue;
                  case bCount:  SetCount(nBtn); continue;
                  case bMult:   nMult = !nMult;
                        if ( nPolar ) {         // Added Jul 16, 1998
                            // make sure these two data consistant
                            dSAngle = ppangle(pCenter,pBase);
                            dRadius = pCenter,pBase;
                        }
                        UpdateDialog(); continue;
                  case bCmd:    nCmd = !nCmd;   UpdateDialog(); continue;
                  case bExp:    nExp = !nExp;
                        wnd_fmark(bExp,0,nExp?MARKON:MARKOFF);
                        continue;
                  case bRot:    nRot = !nRot;
                        wnd_fmark(bRot,0,nRot?MARKON:MARKOFF);
                        continue;
                  case bRect+1:
                        if ( !nPolar ) {        // Added Jul 16, 1998
                            // make sure these two data consistant
                            dSAngle = ppangle(pCenter,pBase);
                            dRadius = pCenter,pBase;
                        }       // Fall down
                  case bRect:
                        nPolar = nBtn-bRect;
                        UpdateMult();
                        continue;
                  case bCancel: nRet = -1;      break;
                  case 0x2308:  // ^H
                  case bHelp:
                        if ( nCallFlag & MODIFYSINGLE )
                                show_help("","CHINSERT");
                        else
                                show_help("","DDINSERT");
                        continue;
                  default:
                        printf("\nCode: %xh",nBtn);
                        continue;
                }
                break;
        }
        CloseDialog();
        return nRet;
}
//
int     ChkBtnState()
{
        if ( identify("wnd_state") ) {
                if ( IsDataOk() ) {
                        if ( bCreate != NOTPRESENT ) wnd_state(bCreate,1,0);
                        if ( bModify != NOTPRESENT ) wnd_state(bModify,1,0);
                }
                else {
                        if ( bCreate != NOTPRESENT ) wnd_state(bCreate,1,1);
                        if ( bModify != NOTPRESENT ) wnd_state(bModify,1,1);
                }
        }
}
//
int     IsDataOk()
{
        if ( szBlock == "" ) {  // No block name
                if ( szName == "" ) return 0;
                if ( !nExp ) return 0;
        }
        if ( nBlock == (nBlkNo-1) )
                if ( szName == "" ) return 0;
        return 1;
}
//
void    SelBName()
{
        nBlock = wnd_fgeto(bBlock,szBlkName,0xf0,8,nBlock);
        if ( nBlock!= (nBlkNo-1) ) szBlock = szBlkName[nBlock];
        else          SetBName();
        wnd_ftext(bBlock,prints("%-*s",FBLKSIZE,szBlock),DATACOLOR);
        ChkBtnState();
}
//
void    SetBName()
{
        szBlock = upper(wnd_fgets(bBlock,FBLKSIZE,0x44e,szBlock));
        wnd_ftext(bBlock,prints("%-*s",FBLKSIZE,szBlock),DATACOLOR);
        nBlock = search(szBlock,szBlkName,nBlkNo);
        if ( nBlock == -1 ) {   // Not found, must be external drawing
                nBlock = nBlkNo-1;
                if ( szName == "" )
                        szName = fsearch(left$(szBlock,8)+".WRK");
                if ( szName == "" )
                        szName = fsearch(left$(szBlock,8)+".DWG");
                if ( szName == "" )
                        SetName();
        }
        wnd_ftext(bName,prints("%-*s",FNAMESIZE,szName),DATACOLOR);
        ChkBtnState();
}
//
void    SetName()
{
        szName = getfile(szName,"WRK;*.DWG",0x4082,
                "Select Drawing File to Insert");
        wnd_ftext(bName,prints("%-*s",FNAMESIZE,szName),DATACOLOR);
        if ( szName == "" ) {   // No External Drawing ...
                if ( nBlock == (nBlkNo-1) )  { // External Drawing!
                        szBlock = "";   // No block now!
                        wnd_ftext(bBlock,prints("%-*s",FBLKSIZE,szBlock),DATACOLOR);
                }
                return;
        }
        if ( szBlock == "" ) {
                szBlock = takename(szName)-".WRK"-".DWG";
                nBlock = search(szBlock,szBlkName,nBlkNo);
                if ( nBlock == -1 ) nBlock = nBlkNo - 1;
                wnd_ftext(bBlock,prints("%-*s",FBLKSIZE,szBlock),DATACOLOR);
        }
        if ( nBlock != (nBlkNo-1) ) {
                if ( !Confirm(szText[28]/*"Block already exist!"*/,
                        szText[29]/*"Do you want to redefine it ? "*/)) {
                        szBlock = takename(szName)-".WRK"-".DWG";
                        nBlock = search(szBlock,szBlkName,nBlkNo);
                        if ( nBlock != -1 ) szBlock = "";
                        nBlock = nBlkNo - 1;
                }
        }
        wnd_ftext(bBlock,prints("%-*s",FBLKSIZE,szBlock),DATACOLOR);
        ChkBtnState();
}
//
void    SetAngle(int nBtn)
{
        dAngle = wnd_fgetd(nBtn,8,0x2e0,dAngle);
        wnd_ftext(nBtn, prints(FMTVALUE,dAngle),DATACOLOR);
}
//
void    SetBAngle(int nBtn)
{
        dBAngle = wnd_fgetd(nBtn,8,0x2e0,dBAngle);
        wnd_ftext(nBtn, prints(FMTVALUE,dBAngle),DATACOLOR);
}
//
void    SetSAngle(int nBtn)
{
        dSAngle = wnd_fgetd(nBtn,8,0x2e0,dSAngle);
        wnd_ftext(nBtn, prints(FMTVALUE,dSAngle),DATACOLOR);
        pBase = polar(pCenter,dRadius,dSAngle);
        UpdateBasic();
}
//
void    SetRadius(int nBtn)
{
        dRadius = wnd_fgetd(nBtn,8,0x2e0,dRadius);
        wnd_ftext(nBtn, prints(FMTVALUE,dRadius),DATACOLOR);
        pBase = polar(pCenter,dRadius,dSAngle);
        UpdateBasic();
}
//
void    SetSpan(int nBtn)
{
        dSpan = mod(wnd_fgetd(nBtn,8,0x2e0,dSpan),360);
        if ( dSpan == 0. ) dSpan = 360.;
        wnd_ftext(nBtn, prints(FMTVALUE,dSpan),DATACOLOR);
}
//
void    SetScale(int nBtn)
{
        double  dScale;
        dScale = (nBtn==bScale)?pScale.x:pScale.y;
        dScale = wnd_fgetd(nBtn,8,0x2e0,dScale);
        if ( dScale == 0. ) dScale = 1.;
        wnd_ftext(nBtn, prints(FMTVALUE,dScale),DATACOLOR);
        ((nBtn==bScale)?pScale.x:pScale.y) = dScale;
}
//
void    SetPos(int nBtn)
{
        double  dval;
        dval = (nBtn==bBase)?pBase.x:pBase.y;
        dval = wnd_fgetd(nBtn,8,0x2e0,dval);
        wnd_ftext(nBtn, prints(FMTVALUE,dval),DATACOLOR);
        ((nBtn==bBase)?pBase.x:pBase.y) = dval;
        dSAngle = ppangle(pCenter,pBase);
        dRadius = pCenter,pBase;
        UpdatePolar();
}
//
void    SetCenter(int nBtn)
{
        double  dval;
        dval = (nBtn==bCenter)?pCenter.x:pCenter.y;
        dval = wnd_fgetd(nBtn,8,0x2e0,dval);
        wnd_ftext(nBtn, prints(FMTVALUE,dval),DATACOLOR);
        ((nBtn==bCenter)?pCenter.x:pCenter.y) = dval;
        dSAngle = ppangle(pCenter,pBase);
        dRadius = pCenter,pBase;
        UpdatePolar();
}
//
void    SetSpace(int nBtn)
{
        double  dval;
        dval = (nBtn==bSpace)?pSpace.x:pSpace.y;
        dval = wnd_fgetd(nBtn,8,0x2e0,dval);
        wnd_ftext(nBtn, prints(FMTVALUE,dval),DATACOLOR);
        ((nBtn==bSpace)?pSpace.x:pSpace.y) = dval;
}
//
void    SetRow(int nBtn)
{
        nRow = wnd_fgetd(nBtn,5,0x2e0,nRow);
        if ( nRow <= 0 ) nRow = 1;
        wnd_ftext(nBtn, prints("%5d",nRow),DATACOLOR);
}
//
void    SetCol(int nBtn)
{
        nCol = wnd_fgetd(nBtn,5,0x2e0,nCol);
        if ( nCol <= 0 ) nCol = 1;
        wnd_ftext(nBtn, prints("%5d",nCol),DATACOLOR);
}
//
void    SetCount(int nBtn)
{
        nCount = wnd_fgetd(nBtn,5,0x2e0,nCount);
        if ( nCount <= 0 ) nCount = 1;
        wnd_ftext(nBtn, prints("%5d",nCount),DATACOLOR);
}
//
//
void    OpenDialog()
{
        int     rBase, cBase, cPos, rPos;
        int     rInc=0x0801;
        int     nVp[5];
        int     nTitle;

        nTitle = 0;
        if ( nCallFlag & (MODIFYSINGLE|MODIFYONLY) ) nTitle = 1;
        widMain = wndopen(nMain,szText[nTitle]/*"Block/Drawing Insertion Dialog"*/,0x3f1f,nMain[5]);
        rBase = 0x0800;
        cBase = 1;
        bBlock = WndEntry(rBase,       cBase,szText[2]/*"Block Name: "*/,0x100|FBLKSIZE,1);
   if ( !(nCallFlag & (MODIFYSINGLE|MODIFYONLY)) )
        bExp = WndCheckBox(rBase,32,szText[4]/*" Exploded"*/,nSatrb);
        rBase += 0x0801;
        bName = WndEntry(rBase,cBase,szText[3]/*" File Name: "*/,0x100|FNAMESIZE,0);

        rPos = 0x0803;
        cPos = 2;

        rBase = rPos;
        cBase = cPos;
        WndGroup(rBase,cBase-1,28,5|0xfa00,szText[5]/*" Insertion Basic "*/,nSatrb);
        rBase += 0x0401;
        WndText(rBase,cBase,szText[6]/*" Pos. : "*/);
        cBase += 8;
        WndGroup(rBase-1,cBase,19,1,"",nSatrb);
        bBase = WndEntry(rBase|0x0100,cBase,"",0x108,0);
               WndEntry(rBase|0x0100,cBase+9,"",0x108,1);

        rBase += 0x0801;
        cBase = cPos;
        WndText(rBase,cBase,szText[7]/*" Scale: "*/);
        cBase += 8;
        WndGroup(rBase-1,cBase,17,1,"",nSatrb);
        bScale = WndEntry(rBase|0x100,cBase,"",0x108,0);
                 WndEntry(rBase|0x100,cBase+9,"",0x108,0);

        rBase += 0x0801;
        cBase = cPos;
        WndText(rBase,cBase,szText[8]/*" Angle: "*/);
        cBase += 8;
        WndGroup(rBase-1,cBase,8,1,"",nSatrb);
        bAngle = WndEntry(rBase|0x100,cBase,"",0x108,0);

        rBase = rPos;
        cBase = 32;
        WndGroup(rBase,cBase-1,16,3|0xfa00,"",nSatrb);
        bMult= WndCheckBox(rBase,cBase,szText[9]/*" Multiple "*/,nSatrb);
        rBase += 0x0401;
        bRect=WndRadioBox(rBase,       cBase,szText[10]/*"Rectangular"*/,nSatrb);
              WndRadioBox(rBase+0x0401,cBase,szText[11]/*"Polar"*/,nSatrb);

        rPos  = 0x0809;
        cPos  = 2;
        rBase = rPos;
        cBase = cPos;
        WndGroup(rBase,cBase-1,28,5|0xfa00,szText[12]/*" Rectangular Array "*/,nSatrb);
        rBase += 0x0801;
        cBase = cPos;
        WndText(rBase,cBase,szText[13]/*"Column: "*/);
        cBase += 8;
        WndGroup(rBase-1,cBase,5,1,"",nSatrb);
        bCol = WndEntry(rBase|0x0100,cBase,"",0x105,0);
        WndText(rBase,cBase+6,szText[14]/*" Row: "*/);
        WndGroup(rBase-1,cBase+12,5,1,"",nSatrb);
        bRow = WndEntry(rBase|0x0100,cBase+12,"",0x105,0);

        rBase += 0x0801;
        cBase = cPos;
        WndText(rBase,cBase,szText[15]/*" Space: "*/);
        cBase += 8;
        WndGroup(rBase-1,cBase,17,1,"",nSatrb);
        bSpace = WndEntry(rBase|0x100,cBase,"",0x108,0);
                 WndEntry(rBase|0x100,cBase+9,"",0x108,0);

        rBase += 0x0801;
        cBase = cPos;
        WndText(rBase,cBase,szText[16]/*" Angle: "*/);
        cBase += 8;
        WndGroup(rBase-1,cBase,8,1,"",nSatrb);
        bBAngle = WndEntry(rBase|0x100,cBase,"",0x108,0);

        rPos = 0x0008;
        cPos = 32;

   if ( !(nCallFlag & (MODIFYSINGLE|MODIFYONLY)) )
        bCmd = WndCheckBox(rPos,cPos,szText[31]/*"Command line parameters"*/,nSatrb);

        rPos = 0x0809;
        cPos = 32;

        rBase = rPos;
        cBase = cPos;
        WndGroup(rBase,cBase-1,28,8|0xfa00,szText[17]/*" Porlar Array "*/,nSatrb);
        rBase += 0x0801;
        cBase  = cPos;
        WndText(rBase,cBase,szText[18]/*"Center: "*/);
        cBase += 8;
        WndGroup(rBase-1,cBase,19,1,"",nSatrb);
        bCenter = WndEntry(rBase|0x0100,cBase,"",0x108,0);
                  WndEntry(rBase|0x0100,cBase+9,"",0x108,1);

        rBase += 0x0801;
        cBase  = cPos;
        WndText(rBase,cBase,szText[19]/*"Radius: "*/);
        cBase += 8;
        WndGroup(rBase-1,cBase,8,1,"",nSatrb);
        bRad  = WndEntry(rBase|0x100,cBase,"",0x108,0);

        rBase += 0x0801;
        cBase  = cPos;
        WndText(rBase,cBase,szText[20]/*" Start: "*/);
        cBase += 8;
        WndGroup(rBase-1,cBase,8,1,"",nSatrb);
        bSAngle= WndEntry(rBase|0x100,cBase,"",0x108,0);

        rBase += 0x0801;
        cBase  = cPos;
        WndText(rBase,cBase,szText[21]/*"  Span: "*/);
        cBase += 8;
        WndGroup(rBase-1,cBase,8,1,"",nSatrb);
        bSpan = WndEntry(rBase|0x100,cBase,"",0x108,0);

        rBase += 0x0801;
        cBase  = cPos;
        WndText(rBase,cBase,szText[22]/*" Count: "*/);
        cBase += 8;
        WndGroup(rBase-1,cBase,5,1,"",nSatrb);
        bCount = WndEntry(rBase|0x100,cBase,"",0x105,0);
        bRot   = WndCheckBox(rBase,cBase+8,szText[23]/*" Rotated"*/,nSatrb);

        rBase = 0x0404;
        cBase = 50;
        rInc = 0x0002;
        bPick   = WndButton(rBase+rInc*0,cBase,0x108,szBtn[6]/*"Sam&ple"*/);
    if ( IsWin )
        bHelp   = WndButton(rBase+rInc*1,cBase,0x108,szBtn[2]/*"&Help"*/);

        rBase = rPos+0x0402+5; // +0x0801;
        cBase = 4;
        if ( nCallFlag & (MODIFYONLY|MODIFYSINGLE) )
            bModify = WndButton(rBase,cBase,0x108,szBtn[0]/*"&OK"*/);
        else
            bCreate = WndButton(rBase,cBase,0x108,szBtn[4]/*"&Create"*/);
        bCancel = WndButton(rBase,cBase+14,0x108,szBtn[1]/*"&Cancel"*/);

}
//
void    UpdateDialog()
{
   if ( !(nCallFlag & (MODIFYSINGLE|MODIFYONLY)) ) {
        wnd_fmark(bExp,0,nExp?MARKON:MARKOFF);
        wnd_fmark(bCmd,0,nCmd?MARKON:MARKOFF);
   }
        wnd_ftext(bName,prints("%-*s",FNAMESIZE,szName),DATACOLOR);
        wnd_ftext(bBlock,prints("%-*s",FBLKSIZE,szBlock),DATACOLOR);
        UpdateBasic();
        UpdateMult();
        ChkBtnState();
}
//
void    UpdateBasic()
{
        int     nTc=DATACOLOR;

        if ( !(nCallFlag & (MODIFYSINGLE|MODIFYONLY)) ) {
                if ( nCmd ) nTc = DATACOLOR1;
                SetBtnState(bBase,bAngle,nCmd?1:0);
        }
        if ( pScale.x == 0 ) pScale.x = 1;
        if ( pScale.y == 0 ) pScale.y = 1;
        wnd_ftext(bBase,   prints(FMTVALUE,pBase.x),nTc);
        wnd_ftext(bBase+1, prints(FMTVALUE,pBase.y),nTc);
        wnd_ftext(bScale, prints(FMTVALUE,pScale.x),nTc);
        wnd_ftext(bScale+1,prints(FMTVALUE,pScale.y),nTc);
        wnd_ftext(bAngle, prints(FMTVALUE,dAngle),nTc);
}
//
void    UpdateMult()
{
        int     nState;
        wnd_fmark(bMult,0,nMult?MARKON:MARKOFF);
        nState = nMult;
        if ( !(nCallFlag & (MODIFYSINGLE|MODIFYONLY)) )
                if ( nCmd ) nState = 0;
        if ( nState ) {
                SetBtnState(bRect,bRect+1,0);
                SetRadioMark(bRect,bRect+1,nPolar);
                SetBtnState(bCol,bBAngle,nPolar?1:0);
                SetBtnState(bCenter,bRot,nPolar?0:1);
        }
        else {
                SetBtnState(bRect,bRect+1,nMult?0:1);
                SetRadioMark(bRect,bRect+1,nPolar);
                SetBtnState(bCol,bBAngle,1);
                SetBtnState(bCenter,bRot,1);
        }
        UpdateRect();
        UpdatePolar();
}
//
void    UpdateRect()
{
        int     nTc=DATACOLOR;
        if ( !nMult || nPolar ) nTc = DATACOLOR1;
        if ( !(nCallFlag & (MODIFYSINGLE|MODIFYONLY)) )
                if ( nCmd ) nTc = DATACOLOR1;
        wnd_ftext(bRow,     prints("%5d",nRow),nTc);
        wnd_ftext(bCol,     prints("%5d",nCol),nTc);
        wnd_ftext(bSpace,   prints(FMTVALUE,pSpace.x),nTc);
        wnd_ftext(bSpace+1, prints(FMTVALUE,pSpace.y),nTc);
        wnd_ftext(bBAngle,  prints(FMTVALUE,dBAngle),nTc);
}
//
void    UpdatePolar()
{
        int     nState;
        int     nTc=DATACOLOR;
        if ( !nMult || !nPolar ) nTc = DATACOLOR1;
        if ( !(nCallFlag & (MODIFYSINGLE|MODIFYONLY)) )
                if ( nCmd ) nTc = DATACOLOR1;
        wnd_ftext(bCount,   prints("%5d",nCount),nTc);
        wnd_ftext(bCenter,  prints(FMTVALUE,pCenter.x),nTc);
        wnd_ftext(bCenter+1,prints(FMTVALUE,pCenter.y),nTc);
        wnd_ftext(bSpan,    prints(FMTVALUE,dSpan),nTc);
        wnd_ftext(bSAngle,  prints(FMTVALUE,dSAngle),nTc);
        wnd_ftext(bRad,     prints(FMTVALUE,dRadius),nTc);
        wnd_fmark(bRot,0,nRot?MARKON:MARKOFF);
}
//
void    CloseDialog()
{
        if ( IsWin ) {
            if ( wnd_getsize(nMain) ) {
                put_profile(SECTIONNAME,MAINWINDOW,
                        prints("%xh,%xh,%xh,%xh,%xh",nMain[0],nMain[1],
                                nMain[2],nMain[3],nMain[4]));
            }
        }
        wndclose();
        SaveSetFile();
}
//
//      Some useful routines
//
void    SetRadioMark(int bStart, int bEnd, int which)
{
        int     i;
        for(i=bStart;i<=bEnd;i++)
                wnd_fmark(i,1,((which+bStart)==i)?MARKON:MARKOFF);
}
//
void    SetBtnState(int bStart, int bEnd, int state)
{
        int     i;
    if ( identify("wnd_state") )        // So it will works for DOS version
        for(i=bStart;i<=bEnd;i++)
                wnd_state(i,1,state);
}
#define OBM_COMBO       "#32738"
//
void    WndText(int row, int col, char str[])
{
        wnd_loct(row,col);
        put3Dstr(str,nSatrb);
}
//
int     WndEntry(int row, int col, char str[],int fsize,int combo)
{
        int     bID;

        wnd_loct(row,col);
        put3Dstr(str,nSatrb);
        bID = wnd_field(0,IsWin?0x1f1:0x21f1,fsize);
        if ( combo ) {
                wnd_rloct(0xfe00,0x0600|(fsize&0xff));
                if ( IsWin) wnd_runit(12);
                if ( identify("cb_empty") ) {   // Supported only after this version
                     wnd_icon(1,0xA77,0x101,OBM_COMBO);
                     wnd_runit(-1);
                     wnd_rloct(0x0200,0x0002);  // locate back
                }
                else {
                     wnd_field(0,IsWin?0x277:0x370,0x101);
                     wnd_runit(IsWin?-1:16);
                     wnd_ftext(bID+1,"",IsWin?0x277:0x270);
                     wnd_rloct(0x0200,0x0201);  // locate back
                }
        }
        return bID;
}
//
int     WndButton(int r,int c, int bsize, char text[])
{
        int     i;
        if ( nSupport )         // Having direct support ?
                return wnd_button(r,c,bsize,0x1277,text);
        wnd_loct(r,c);
        i = wnd_field(0,IsWin?0x1277:0x370,bsize);
        wnd_puts(text,0x270);
        return i;
}
//
//      WndGroup() -- Create Groupping window effect
//
void    WndGroup(int r,int c,int width, int height, char text[], int color)
{
        wnd_loct(r,c);
        wnd_rloct(height,0);
        if ( IsWin )
                wnd_field(0,0x277,(height<<8)|width);
        else
                wnd_field(0,((color&0xff)^0x08)|0x200,(height<<8)|width);
        wnd_itemno(1,-1);
        wnd_rloct(-height,2);
        if ( !IsWin )
                wnd_rloct(0xfc00,0);
        put3Dstr(text,color);
        wnd_loct(r,c);        // Reset row offset, fix DOS version bug
}
//
//      put3Dstr() -- Drawing 3D text
//
void    put3Dstr(char str[],int atrb)
{
        int     nLen;
        if ( nSupport ) {
                wnd_puts(str,atrb|0x300);
                return;
        }
        nLen=len(str);
        if ( nLen ) {
                wnd_rloct(0x100,IsWin?0x100:0);
                wnd_puts(str,(atrb^0x0f));
                wnd_rloct(0xff00,-nLen);
                wnd_puts(str,(atrb&0xff0f)|0x8000);
        }
}
//
//      WndCheckBox() -- Create window check box
//
int     WndCheckBox(int r,int c,char text[],int atrb)
{
        int     bID;
        if ( nSupport )
                return wnd_rcbox(r,c,text,0);
        wnd_loct(r,c);
        bID = wnd_field(1,0x1f0,0x102);
        wnd_rloct(0,2);
        if ( IsWin )  atrb |= 0x800;
        wnd_puts(text,atrb);
        return bID;
}
//
//      WndRadioBox() -- Create window radio box
//
int     WndRadioBox(int r,int c,char text[],int atrb)
{
        int     bID;
        if ( nSupport )
                return wnd_rcbox(r,c,text,0x8000);
        wnd_loct(r,c);
        bID = wnd_field(1,0x1f0,0x102);
        wnd_rloct(0,2);
        if ( IsWin ) atrb |= 0x800;
        wnd_puts(text,atrb);
        wnd_fmark(bID,1,IsWin?1:MARKON);     // Force to change state
        wnd_fmark(bID,1,IsWin?0:MARKOFF);
        return bID;
}
//
int     Confirm(char szTitle[], char szMsg[])
{
        if ( IsWin ) {
                if ( MessageBox(szMsg,szTitle,0x0024) == 7 )
                        return 0;
        }
        else {
                if ( wnd2btn(szTitle,szMsg,"   Yes  ","   No   ") != 1 )
                        return 0;
        }
        return 1;

}
//
//      Check for program interface
//
int     CheckCall()
{
        char    szBuf[256];
        nCallFlag = 0;  // Normal invocation
        exbuf_io(1,szBuf,sizeof(szBuf));
        if ( left$(szBuf,10) == "%%DDINSERT" ) {
                nCallFlag = BEINGCALLED;
                szBuf -= "%%DDINSERT";
                if ( szBuf == " -MODIFY" )
                        nCallFlag |= MODIFYONLY;
                else if ( szBuf == " -MODIFYSINGLE" )
                        nCallFlag |= MODIFYSINGLE;
                szBuf = "";
                exbuf_io(0,szBuf,128);     // Must clear it!
        }
}

#define GENERAL         "General Setup"
#define HATCHFILE       "HatchPatternFile"
int     Initialize()
{
        char    items[1][64];
        long    lSize, lDate;
        char    szBuf[256];

        // Determine the main dialog window position
        if ( IsWin ) {  // Only in Windows version
                szBuf = get_profile(SECTIONNAME,MAINWINDOW);
                if ( szBuf != "" ) {    // Yes, not the first time
                        int     nWidth;
                        int     nHeight;
                        nWidth=nMain[3]-nMain[2];
                        nHeight=nMain[1]-nMain[0];
                        sscanf(szBuf,"%xh, %xh, %xh, %xh, %xh",&nMain[0],&nMain[1],
                                &nMain[2],&nMain[3],&nMain[4]);
                        // Since we are using fix window
                        nMain[1] = nMain[0]+nHeight;
                        nMain[3] = nMain[2]+nWidth;
                        nTatrb = nMain[4]&0xff;
                        nMain[5] = 0;
                }
        }
        if ( !IsWin )
                nTatrb = 0x2f1;

        GetBlkTable();

        szSetFile = fullname("DDINSERT.SET",get_syspath(0));
        LoadSetFile();
        nBlock = search(szBlock,szBlkName,nBlkNo);
        if ( nBlock == -1 ) {
                nBlock = nBlkNo - 1;
                szBlock = "";
                if ( !fexist(szName) ) szName = "";
                if ( szName != "" ) {
                        szBlock = takename(szName)-".WRK"-".DWG";
                        nBlock = search(szBlock,szBlkName,nBlkNo);
                        if ( nBlock == -1 ) nBlock = nBlkNo - 1;
                }
        }
        else {
                szName = "";
        }
}
//
void    GetBlkTable()
{
        memset(szBlkName,0);
        nBlkNo = 0;
        s1 = "";
        while((s1=tbl_next(s1,3))!="") szBlkName[nBlkNo++] = s1;
        if ( nBlkNo>1 ) sort(szBlkName,nBlkNo,0);
        szBlkName[nBlkNo++] = szText[30] /*"<Drawing>" */;
}
//
//      Load current setting from local set file
//
BOOL    LoadSetFile()
{
        int     hdl;
        if ( !(hdl=fopen(szSetFile,"r")) )
                return FALSE;
        fget(hdl,szBlock,szName);
        fget(hdl,&nMult,&nPolar,&nExp);
        fget(hdl,&pBase,&pScale,&dAngle,&dBAngle);
        fget(hdl,&nRow, &nCol, &pSpace, &pCenter);
        fget(hdl,&dSpan,&dSAngle,&dRadius,&nCount,&nRot,&nCmd);
        fclose(hdl);
        return TRUE;
}
//
//      Save current setting to local set file
//
BOOL    SaveSetFile()
{
        int     hdl;

        if ( !(hdl=fopen(szSetFile,"w")) )
                return FALSE;
        fput(hdl,szBlock,szName);
        fput(hdl,nMult,nPolar,nExp);
        fput(hdl,pBase,pScale,dAngle,dBAngle);
        fput(hdl,nRow, nCol, pSpace, pCenter);
        fput(hdl,dSpan,dSAngle,dRadius,nCount,nRot,nCmd);
        fclose(hdl);
        return TRUE;
}

char    szAsciiText[][64]={         // Default English Display Text
/*  0 */   "Block/Drawing Insertion Dialog",
/*  1 */   "Block Insertion Modification Dialog",
/*  2 */   "Block Name: ",
/*  3 */   " File Name: ",
/*  4 */   " Exploded",
/*  5 */   " Insertion Basic ",
/*  6 */   " Pos. : ",
/*  7 */   " Scale: ",
/*  8 */   " Angle: ",
/*  9 */   " Multiple ",
/* 10 */   "Rectangular",
/* 11 */   "Circular",
/* 12 */   " Rectangular Array ",
/* 13 */   "Column: ",
/* 14 */   " Row: ",
/* 15 */   " Space: ",
/* 16 */   " Angle: ",
/* 17 */   " Circular Array ",
/* 18 */   "Center: ",
/* 19 */   "Radius: ",
/* 20 */   " Start: ",
/* 21 */   "  Span: ",
/* 22 */   " Count: ",
/* 23 */   " Rotated",
/* 24 */   "\nSelect the block instance to modify: ",
/* 25 */   "\nPlease designate the base insertion point: ",
/* 26 */   "\nPlease designate the rotation center point: ",
/* 27 */   "\nPlease select the block instance to sample: ",
/* 28 */   "Block already exist!",
/* 29 */   "Do you want to redefine it ? ",
/* 30 */   "<Drawing>",
/* 31 */   "Command line parameters",
};

char    szBig5Text[][64]={         // Default Chinese Big-5 Display Text
/*  0 */   "ϲդιɴJާ@ܵ",
/*  1 */   "ϲմJާ@ܵ",
/*  2 */   "  ϲզW: ",
/*  3 */   "  ɸ|: ",
/*  4 */   "z}J",
/*  5 */   " 򥻴J]w",
/*  6 */   "  m: ",
/*  7 */   "  : ",
/*  8 */   "ਤ: ",
/*  9 */   "hJ ",
/* 10 */   "x}",
/* 11 */   "",
/* 12 */   " xΰ}C ",
/* 13 */   "  : ",
/* 14 */   "C: ",
/* 15 */   "  Z: ",
/* 16 */   "C: ",
/* 17 */   " }C ",
/* 18 */   "I: ",
/* 19 */   "  b|: ",
/* 20 */   "  l: ",
/* 21 */   "  i: ",
/* 22 */   "  Ӽ: ",
/* 23 */   "H",
/* 24 */   "\nпܭn諸ϲմJ: ",
/* 25 */   "\nЫwǴJI: ",
/* 26 */   "\nЫwI: ",
/* 27 */   "\nпܭnѦŪϲմJ: ",
/* 28 */   "ҫwϲդwMsb!",
/* 29 */   "znswq ? ",
/* 30 */   "<>",
/* 31 */   "HO覡JJ",
};

char    szGB2312Text[][64]={    // Default Chinese GB2312 Display Text
/*  0 */   "ͼ鼰ͼԻ",
/*  1 */   "ͼĲԻ",
/*  2 */   "  ͼ: ",
/*  3 */   "  ͼ·: ",
/*  4 */   "",
/*  5 */   " 趨",
/*  6 */   "  λ: ",
/*  7 */   "  : ",
/*  8 */   "ת: ",
/*  9 */   "ز ",
/* 10 */   "ʽ",
/* 11 */   "״ʽ",
/* 12 */   "  ",
/* 13 */   "  : ",
/* 14 */   ": ",
/* 15 */   "  : ",
/* 16 */   "нǶ: ",
/* 17 */   " ״ ",
/* 18 */   "ĵ: ",
/* 19 */   "  뾶: ",
/* 20 */   "  ʼ: ",
/* 21 */   "  Ž: ",
/* 22 */   "  : ",
/* 23 */   "֮ת",
/* 24 */   "\nѡҪĵͼ: ",
/* 25 */   "\nָ׼: ",
/* 26 */   "\nָ״ĵ: ",
/* 27 */   "\nѡҪȡͼ: ",
/* 28 */   "ָͼȻ!",
/* 29 */   "Ҫ¶ ? ",
/* 30 */   "<ͼ>",
/* 31 */   "ָзʽ",
};
//
//      The following Japenese messages are translated by _____________.
//
char    szSJISText[][64]={    // Default Japenese SJIS Display Text
/*  0 */   "ubNƐ}ʑ}̏ڍאݒ",
/*  1 */   "ubN}̏ڍאݒ",
/*  2 */   "ubN: ",
/*  3 */   "t@C: ",
/*  4 */   "",
/*  5 */   "} ",
/*  6 */   "W: ",
/*  7 */   ": ",
/*  8 */   "px: ",
/*  9 */   " ",
/* 10 */   "`",
/* 11 */   "~`",
/* 12 */   "`z ",
/* 13 */   "s: ",
/* 14 */   ": ",
/* 15 */   "Ԋu: ",
/* 16 */   "px: ",
/* 17 */   "~`z ",
/* 18 */   "S: ",
/* 19 */   "a: ",
/* 20 */   "n_: ",
/* 21 */   "S͈: ",
/* 22 */   ": ",
/* 23 */   "]",
/* 24 */   "\nCubNIĉ: ",
/* 25 */   "\n}_w肵ĉ: ",
/* 26 */   "\n]̒Sw肵ĉ: ",
/* 27 */   "\nTvɂubNIĉ: ",
/* 28 */   "ubN͊ɑ݂܂B",
/* 29 */   "Ē`܂H ",
/* 30 */   "<}>",
/* 31 */   "ڍאݒ",
};
//
//      The following Korean messages are translated by _____________.
//
char    szKoreanText[][64]={    // Default Korean Display Text
/*  0 */   "Block/Drawing Insertion Dialog",
/*  1 */   "Block Insertion Modification Dialog",
/*  2 */   "Block Name: ",
/*  3 */   " File Name: ",
/*  4 */   " Exploded",
/*  5 */   " Insertion Basic ",
/*  6 */   " Pos. : ",
/*  7 */   " Scale: ",
/*  8 */   " Angle: ",
/*  9 */   " Multiple ",
/* 10 */   "Rectangular",
/* 11 */   "Circular",
/* 12 */   " Rectangular Array ",
/* 13 */   "Column: ",
/* 14 */   " Row: ",
/* 15 */   " Space: ",
/* 16 */   " Angle: ",
/* 17 */   " Circular Array ",
/* 18 */   "Center: ",
/* 19 */   "Radius: ",
/* 20 */   " Start: ",
/* 21 */   "  Span: ",
/* 22 */   " Count: ",
/* 23 */   " Rotated",
/* 24 */   "\nSelect the block instance to modify: ",
/* 25 */   "\nPlease designate the base insertion point: ",
/* 26 */   "\nPlease designate the rotation center point: ",
/* 27 */   "\nPlease select the block instance to sample: ",
/* 28 */   "Block already exist!",
/* 29 */   "Do you want to redefine it ? ",
/* 30 */   "<Drawing>",
/* 31 */   "Command line parameters",
};

char    szAsciiBtn[][10]={         // Default English Display Text
/*  0 */   "&OK",
/*  1 */   "&Cancel",
/*  2 */   "&Help",
/*  3 */   "&Quit",
/*  4 */   "C&reate",
/*  5 */   "&Modify",
/*  6 */   "Sam&ple",
/*  7 */   "Pre&view",
};

char    szBig5Btn[][10]={         // Default English Display Text
/*  0 */   "Tw[&O]",
/*  1 */   "[&C]",
/*  2 */   "[&H]",
/*  3 */   "hX[&Q]",
/*  4 */   "Ы[&R]",
/*  5 */   "[&M]",
/*  6 */   "ѷ[&P]",
/*  7 */   "w[&V]",
};

char    szGB2312Btn[][10]={         // Default English Display Text
/*  0 */   "ȷ[&O]",
/*  1 */   "ȡ[&C]",
/*  2 */   "˵[&H]",
/*  3 */   "˳[&Q]",
/*  4 */   "[&R]",
/*  5 */   "[&M]",
/*  6 */   "[&P]",
/*  7 */   "Ԥ[&V]",
};

char    szSJISBtn[][10]={         // Default English Display Text
/*  0 */   "&OK",
/*  1 */   "ݾ[&C]",
/*  2 */   "[&H]",
/*  3 */   "I[&Q]",
/*  4 */   "쐬[&R]",
/*  5 */   "C[&M]",
/*  6 */   "[&P]",
/*  7 */   "\\[&V]",
};

char    szKoreanBtn[][10]={         // Default English Display Text
/*  0 */   "&OK",
/*  1 */   "&Cancel",
/*  2 */   "&Help",
/*  3 */   "&Quit",
/*  4 */   "C&reate",
/*  5 */   "&Modify",
/*  6 */   "Sam&ple",
/*  7 */   "Pre&view",
};

char    szDOSAsciiBtn[][10]={         // Default English Display Text
/*  0 */   "   OK   ",
/*  1 */   " Cancel ",
/*  2 */   "  Help  ",
/*  3 */   "  Quit  ",
/*  4 */   " Create ",
/*  5 */   " Modify ",
/*  6 */   " Sample ",
/*  7 */   " Preview",
};

char    szDOSBig5Btn[][10]={         // Default English Display Text
/*  0 */   "  Tw  ",
/*  1 */   "    ",
/*  2 */   "    ",
/*  3 */   "  hX  ",
/*  4 */   "  Ы  ",
/*  5 */   "    ",
/*  6 */   "  ѷ  ",
/*  7 */   "  w  ",
};

char    szDOSGB2312Btn[][10]={       // Default English Display Text
/*  0 */   "  ȷ  ",
/*  1 */   "  ȡ  ",
/*  2 */   "  ˵  ",
/*  3 */   "  ˳  ",
/*  4 */   "    ",
/*  5 */   "    ",
/*  6 */   "    ",
/*  7 */   "  Ԥ  ",
};

char    szDOSSJISBtn[][10]={         // Default English Display Text
/*  0 */   "   OK   ",
/*  1 */   "  ݾ ",
/*  2 */   "    ",
/*  3 */   "  I  ",
/*  4 */   "  쐬  ",
/*  5 */   "  C  ",
/*  6 */   "   ",
/*  7 */   "  \\  ",
};

char    szDOSKoreanBtn[][10]={         // Default English Display Text
/*  0 */   "   OK   ",
/*  1 */   " Cancel ",
/*  2 */   "  Help  ",
/*  3 */   "  Quit  ",
/*  4 */   " Create ",
/*  5 */   " Modify ",
/*  6 */   " Sample ",
/*  7 */   " Preview",
};

#define BIG5            0x404
#define GB2312          0x804
#define SJIS            0x411
#define KOREAN          0x412
#define ASCII           0x409

#define TGBIG5          1
#define TGGB2312        2
#define TGSJIS          5
#define TGKOREAN        3

int     SetMsgText()
{
        char    szBuf[80];
        int     switched;

        IsWin = 1;
        switch(1) {  // Test if we are in TwinCAD ...
          case !identify("os_type"):
          case (os_type()<0):   // Must be in DOS version!
                IsWin = 0;
                break;
        }

        if ( IsWin ) {
            szBuf = get_profile("General Setup","AlternateLanguage");
            switched = 0;
            if ( szBuf!="" )
                switched = eval(szBuf);
            setdata(szText,szAsciiText);
            setdata(szBtn,szAsciiBtn);
            if ( switched ) {
                switch(os_language()) {
                  case BIG5:    setdata(szText,szBig5Text);     break;
                  case GB2312:  setdata(szText,szGB2312Text);   break;
                  case SJIS:    setdata(szText,szSJISText);     break;
                  case KOREAN:  setdata(szText,szKoreanText);   break;
                  default:      setdata(szText,szAsciiText);    break;
                }
                switch(os_language()) {
                  case BIG5:    setdata(szBtn,szBig5Btn);     break;
                  case GB2312:  setdata(szBtn,szGB2312Btn);   break;
                  case SJIS:    setdata(szBtn,szSJISBtn);     break;
                  case KOREAN:  setdata(szBtn,szKoreanBtn);   break;
                  default:      setdata(szBtn,szAsciiBtn);    break;
                }
            }
        }
        else {  // Switched or not, depends on TGF type in DOS version
            setdata(szText,szAsciiText);
            setdata(szBtn,szDOSAsciiBtn);
            switch(@tgftype) {
              case TGBIG5:    setdata(szText,szBig5Text);     break;
              case TGGB2312:  setdata(szText,szGB2312Text);   break;
              case TGSJIS:    setdata(szText,szSJISText);     break;
              case TGKOREAN:  setdata(szText,szKoreanText);   break;
              default:        setdata(szText,szAsciiText);    break;
            }
            switch(@tgftype) {
              case TGBIG5:    setdata(szBtn,szDOSBig5Btn);     break;
              case TGGB2312:  setdata(szBtn,szDOSGB2312Btn);   break;
              case TGSJIS:    setdata(szBtn,szDOSSJISBtn);     break;
              case TGKOREAN:  setdata(szBtn,szDOSKoreanBtn);   break;
              default:        setdata(szBtn,szDOSAsciiBtn);    break;
            }
        }
        return 0;
}
